`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Kyushu Institute of Technology
// Engineer: DSP Lab
// 
// Create Date:    10:43:18 10/11/2017 
// Design Name: Neural Network (using backpropagation)
// Module Name:    b3_2 
// Project Name: LSI Design Contest in Okinawa 2018
// Target Devices: 
// Tool versions: 
//
// Description: 
//		Calculation of b3_2, when the select initial signal is active, 
// 	the output will the initial value of b3_2, and when the select update 
//		signal is active, the output will be the new value of b3_2
//	Input: 
//		db3_2	: 32 bits 0000_0000.0000_0000_0000_0000_0000_0000 signed : delta bias3_2
//		clk	: 1 bit
//		reset	: 1 bit : high active
//		select_initial	: 1 bit	: high active
//		select_update	: 1 bit	: high active
//	Output:
//		b3_2	: 32 bits 0000_0000.0000_0000_0000_0000_0000_0000 signed : bias3_2
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module b3_2(clk, reset, db3_2, select_initial, select_update, b3_2);
	
	input clk, reset;
	input select_initial, select_update;
	input signed [31:0] db3_2;
	output signed [31:0] b3_2;
	
	reg  zero = 0;
	reg signed [31:0] b3_2;
	reg signed [31:0] init_b3_2 = 32'b1111_1111_0000_0000_0000_0000_0000_0000;
	reg signed [31:0] net1, net2;
		
	always @(posedge clk)
		begin
			if(reset==1)
				begin
					net1=0;
					net2=0;
					b3_2=0;
				end
			else
				begin
					net1 = select_update? db3_2 : zero;
					net2 = b3_2+ net1;
					b3_2 = select_initial? init_b3_2 : net2;
				end
		end
endmodule
