`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Kyushu Institute of Technology
// Engineer: DSP Lab
// 
// Create Date:    10:47:27 10/09/2017 
// Design Name: Neural Network (using backpropagation)
// Module Name:    dadz 
// Project Name: LSI Design Contest in Okinawa 2018
// Target Devices: 
// Tool versions: 
//
//	Description:
//		Calculation of differential a
// 
// Input:(i=1,2,3 , j=1,2)
//  	a3_j : 32bits : output_layer_output
//  	a2_i : 32bits : hidden_layer_output
//
// Output:(i=1,2,3 , j=1,2)
//  	dadz2_j : 32bits : differential a2_j
//  	dadz3_j : 32bits : differential a3_j
//  	(a : sigmoid function)
//  	da/dz=(1/(1+e^(?z) ))=(a?1)a
// Latency:
//
//	Dependencies: 
// 
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module dadz(clk,res, a,dadz);
   input					clk;
	input					res;
	input signed 	[31:0]		a;
	output signed 	[31:0]		dadz;
	// reg signed 		[31:0]		dadz;
	reg signed		[63:0]		tmp_dadz;
	
	always@(posedge clk or posedge res)
	begin
		if (res == 1'b1)
			tmp_dadz = 0;
		else
			tmp_dadz = (32'b0000_0001_0000_0000_0000_0000_0000_0000 - a) * a;
	end

	assign  dadz = tmp_dadz[55:24];
	
endmodule
