`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Kyushu Institute of Technology
// Engineer: DSP Lab
// 
// Create Date:    13:48:25 09/06/2017 
// Design Name: Neural Network (using backpropagation)
// Module Name:    z2
// Project Name: LSI Design Contest in Okinawa 2018
// Target Devices: 
// Tool versions: 
//
// Description: 
// 	Calculation of 2 input (k1,k2) and 1 output (z2)
// Input:
//		ki			: 32 bits 0000_0000.0000_0000_0000_0000_0000_0000 unsigned 	: input data (i=1,2)
//		b2			: 32 bits 0000_0000.0000_0000_0000_0000_0000_0000 signed 	: bias value
//		w2_i		: 32 bits 0000_0000.0000_0000_0000_0000_0000_0000 unsigned 	: weight value (i=1,2)
//		clk		: 1 bit
//		reset		: 1 bit : high active
// Output: 
//		z2			: 8 bits 0000.0000 : output value
// Latency: 1 clk
// 
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Name : date : what changed
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module z2(clk, reset, k1, k2, w2_1, w2_2, b2, z2);

input clk;
input reset;
input signed [31:0] k1, k2;					//͑w̓ input	from first layer
input signed [31:0] w2_1, w2_2;				//ԑw̏d		weight of hidden layer
input signed [31:0] b2;							//ԑw̃oCAX	bias of hidden layer
output signed [7:0] z2;							//ԑw̏o	 	output of hidden layer

reg signed [63:0] net1, net2;
reg signed [31:0] z2_tmp;

always @(posedge clk)
	begin
	//unsynchronized reset
		if (reset==1)
		begin
			net1=0;
			net2=0;
			z2_tmp=0;
		end
		else 
		begin
			net1 = k1*w2_1;
			net2 = k2*w2_2;
			z2_tmp = net1[55:24] + net2[55:24] + b2;
		end 
	end
 
	assign z2 = z2_tmp[27:20];

endmodule 