`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Kyushu Institute of Technology
// Engineer: DSP Lab
// 
// Create Date:    13:48:25 09/06/2017 
// Design Name: Neural Network (using backpropagation)
// Module Name:    z3
// Project Name: LSI Design Contest in Okinawa 2018
// Target Devices: 
// Tool versions: 
//
// Description: 
// 	Calculation of 3 input (a2) and 1 output (z3)
// Input:
//		a2_i	: 32 bits 0000_0000.0000_0000_0000_0000_0000_0000 unsigned 	: input data (i=1,2,3)
//		b3		: 32 bits 0000_0000.0000_0000_0000_0000_0000_0000 signed		: bias value
//		w3_i	: 32 bits 0000_0000.0000_0000_0000_0000_0000_0000 unsigned 	: weight value (i=1,2,3)
//		clk	: 1 bit
//		reset	: 1 bit : high active
// Output: 
//		z3		: 8 bits 0000.0000 : output value
// Latency: 1 clk
// 
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Name : date : what changed
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module z3(clk, reset, a2_1, a2_2, a2_3, w3_1, w3_2, w3_3, b3, z3);

input clk;
input reset;
input signed [31:0] a2_1, a2_2, a2_3;				//ԑw̓ input from hidden layer
input signed [31:0] w3_1, w3_2, w3_3;				//o͑w̏d		weight of third layer
input signed [31:0] b3;									//o͑w̃oCAX	bias of third layer
output signed [7:0] z3;									//o͑w̏o		output of third layer

reg signed [63:0] net1, net2, net3;
reg signed [31:0] z3_tmp;

always @(posedge clk)
	begin
	//unsynchronized reset
	if (reset==1)
		begin
			net1=0;
			net2=0;
			net3=0;
			z3_tmp=0;
		end
	else
		begin
			net1= a2_1*w3_1;
			net2= a2_2*w3_2;
			net3= a2_3*w3_3;
			z3_tmp = (net1[55:24] + net2[55:24] + net3[55:24] + b3);
		end
	end
assign z3 = z3_tmp[27:20];//

endmodule
